﻿using TravelAgencyBusinessLogic.OfficePackage.HelperModels;
using TravelAgencyBusinessLogic.OfficePackage;
using TravelAgencyContracts.BindingModels;
using TravelAgencyContracts.BusinessLogicsContracts;
using TravelAgencyContracts.StoragesContracts;
using TravelAgencyContracts.ViewModels;
using TravelAgencyContracts.SearchModels;

namespace TravelAgencyBusinessLogic.BusinessLogics
{
	public class ReportLogic : IReportLogic
	{
        private readonly ITourStorage _tourStorage;

        private readonly IPlaceStorage _placeStorage;

        private readonly IGuideStorage _guideStorage;

        private readonly IExcursionGroupStorage _excursionGroupStorage;

        private readonly IExcursionStorage _excursionStorage;

        private readonly AbstractSaveToWord _saveToWord;

        private readonly AbstractSaveToExcel _saveToExcel;

        private readonly AbstractSaveToPdf _saveToPdf;

        public ReportLogic(ITourStorage tourStorage,
                IPlaceStorage placeStorage,
                IGuideStorage guideStorage,
                IExcursionGroupStorage excursionGroupStorage,
                IExcursionStorage excursionStorage,
                AbstractSaveToWord saveToWord,
                AbstractSaveToExcel saveToExcel,
                AbstractSaveToPdf saveToPdf)
        {
            _tourStorage = tourStorage;
            _placeStorage = placeStorage;
            _guideStorage = guideStorage;
            _excursionGroupStorage = excursionGroupStorage;
            _excursionStorage = excursionStorage;

            _saveToWord = saveToWord;
            _saveToExcel = saveToExcel;
            _saveToPdf = saveToPdf;
        }

        public List<ReportTourPlaceViewModel> GetTourPlaces(ReportBindingModel model)
        {
            var result = new List<ReportTourPlaceViewModel>();

            var excursions = _excursionStorage.GetFullList();

            foreach (var tour in model.Tours)
            {
                var record = new ReportTourPlaceViewModel
                {
                    Tour = _tourStorage.GetElement(new TourSearchModel{ Id = tour }),
                    Places = new HashSet<PlaceViewModel>()
                };

                foreach (var excursion in excursions)
                {
                    if (excursion.ExcursionTours.ContainsKey(tour))
                    {
                        var place = _placeStorage.GetElement(new PlaceSearchModel { Id = excursion.PlaceId });
                        record.Places.Add(place);
                    }
                }

                result.Add(record);
            }

            return result;
        }

        public List<ReportTourPeriodViewModel> GetTourPeriod(ReportBindingModel model)
        {
            var result = new List<ReportTourPeriodViewModel>();

            var tours = _tourStorage.GetFilteredList(new TourSearchModel
            {
                UserId = model.UserId,
                DateFrom = model.DateFrom,
                DateTo = model.DateTo
            });

            var excursionGroups = _excursionGroupStorage.GetFullList();

            foreach (var tour in tours)
            {
                var record = new ReportTourPeriodViewModel
                {
                    Tour = tour,
                    ExcursionGroups = new HashSet<ExcursionGroupViewModel>(),
                    Guides = new HashSet<GuideViewModel>(),
                };

                foreach (var excursionGroup in excursionGroups)
                {
                    if (excursionGroup.ExcursionGroupTours.ContainsKey(tour.Id))
                    {
                        record.ExcursionGroups.Add(excursionGroup);
                        var guide = _guideStorage.GetElement(new GuideSearchModel { Id = excursionGroup.GuideId });
                        record.Guides.Add(guide);
                    }
                }

                result.Add(record);
            }

            return result;
        }

        public void SaveTourPlacesToWordFile(ReportBindingModel model)
        {
            _saveToWord.CreateDoc(new WordInfo
            {
                FileStream = model.FileStream,
                Title = "Список мест для посещения по выбранным турам.",
                TourPlaces = GetTourPlaces(model)
            });
        }

        public void SaveTourPlacesToExcelFile(ReportBindingModel model)
        {
            _saveToExcel.CreateReport(new ExcelInfo
            {
                FileStream = model.FileStream,
                Title = "Список мест",
                TourPlaces = GetTourPlaces(model)
            });
        }
    }
}
