﻿using TravelAgencyBusinessLogic.OfficePackage.HelperEnums;
using TravelAgencyBusinessLogic.OfficePackage.HelperModels;

namespace TravelAgencyBusinessLogic.OfficePackage
{
    public abstract class AbstractSaveToExcel
    {
		/// <summary>
		/// Создание отчета
		/// </summary>
		/// <param name="info"></param>
		public void CreateReport(ExcelInfo info)
		{
			CreateExcel(info);

			InsertCellInWorksheet(new ExcelCellParameters
			{
				ColumnName = "A",
				RowIndex = 1,
				Text = info.Title,
				StyleInfo = ExcelStyleInfoType.Title
			});

			MergeCells(new ExcelMergeParameters
			{
				CellFromName = "A1",
				CellToName = "C1"
			});

			uint rowIndex = 2;
			foreach (var tp in info.TourPlaces)
			{
				InsertCellInWorksheet(new ExcelCellParameters
				{
					ColumnName = "A",
					RowIndex = rowIndex,
					Text = $"Тур: {tp.Tour.TourName}.",
					StyleInfo = ExcelStyleInfoType.Text
				});
				rowIndex++;

                InsertCellInWorksheet(new ExcelCellParameters
                {
                    ColumnName = "B",
                    RowIndex = rowIndex,
                    Text = "Места для посещения:",
                    StyleInfo = ExcelStyleInfoType.Text
                });

                MergeCells(new ExcelMergeParameters
                {
                    CellFromName = $"B{rowIndex}",
                    CellToName = $"C{rowIndex}"
                });

                rowIndex++;

                InsertCellInWorksheet(new ExcelCellParameters
                {
                    ColumnName = "B",
                    RowIndex = rowIndex,
                    Text = "Название:",
                    StyleInfo = ExcelStyleInfoType.Text
                });

                InsertCellInWorksheet(new ExcelCellParameters
                {
                    ColumnName = "C",
                    RowIndex = rowIndex,
                    Text = "Адрес:",
                    StyleInfo = ExcelStyleInfoType.Text
                });

                rowIndex++;

                foreach (var place in tp.Places)
				{
					InsertCellInWorksheet(new ExcelCellParameters
					{
						ColumnName = "B",
						RowIndex = rowIndex,
						Text = place.PlaceName,
						StyleInfo = ExcelStyleInfoType.TextWithBorder
					});

					InsertCellInWorksheet(new ExcelCellParameters
					{
						ColumnName = "C",
						RowIndex = rowIndex,
						Text = place.PlaceAddress,
						StyleInfo = ExcelStyleInfoType.TextWithBorder
					});

					rowIndex++;
				}
			}

			SaveExcel(info);
		}

		/// <summary>
		/// Создание excel-файла
		/// </summary>
		/// <param name="info"></param>
		protected abstract void CreateExcel(ExcelInfo info);

		/// <summary>
		/// Добавляем новую ячейку в лист
		/// </summary>
		/// <param name="cellParameters"></param>
		protected abstract void InsertCellInWorksheet(ExcelCellParameters excelParams);

		/// <summary>
		/// Объединение ячеек
		/// </summary>
		/// <param name="mergeParameters"></param>
		protected abstract void MergeCells(ExcelMergeParameters excelParams);

		/// <summary>
		/// Сохранение файла
		/// </summary>
		/// <param name="info"></param>
		protected abstract void SaveExcel(ExcelInfo info);
	}
}