﻿using Microsoft.EntityFrameworkCore;
using TravelAgencyContracts.BindingModels;
using TravelAgencyContracts.SearchModels;
using TravelAgencyContracts.StoragesContracts;
using TravelAgencyContracts.ViewModels;
using TravelAgencyDatabaseImplement.Models;

namespace TravelAgencyDatabaseImplement.Implements
{
    public class ExcursionGroupStorage : IExcursionGroupStorage
    {
        public List<ExcursionGroupViewModel> GetFullList()
        {
            using var context = new TravelAgencyDatabase();
            return context.ExcursionGroups
                    .Include(x => x.User)
                    .Include(x => x.Guide)
                    .Include(x => x.Tours)
                    .ThenInclude(x => x.Tour)
                    .Include(x => x.Places)
                    .ThenInclude(x => x.Place)
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<ExcursionGroupViewModel> GetFilteredList(ExcursionGroupSearchModel model)
        {
            using var context = new TravelAgencyDatabase();
            if (!string.IsNullOrEmpty(model.ExcursionGroupName))
            {
                return context.ExcursionGroups
                    .Include(x => x.User)
                    .Include(x => x.Guide)
                    .Include(x => x.Tours)
                    .ThenInclude(x => x.Tour)
                    .Include(x => x.Places)
                    .ThenInclude(x => x.Place)
                    .Where(x => x.ExcursionGroupName.Contains(model.ExcursionGroupName))
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            if (model.UserId.HasValue)
            {
                return context.ExcursionGroups
                    .Include(x => x.User)
                    .Include(x => x.Guide)
                    .Include(x => x.Tours)
                    .ThenInclude(x => x.Tour)
                    .Include(x => x.Places)
                    .ThenInclude(x => x.Place)
                    .Where(x => x.UserId.Equals(model.UserId))
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            if (model.GuideId.HasValue)
            {
                return context.ExcursionGroups
                    .Include(x => x.User)
                    .Include(x => x.Guide)
                    .Include(x => x.Tours)
                    .ThenInclude(x => x.Tour)
                    .Include(x => x.Places)
                    .ThenInclude(x => x.Place)
                    .Where(x => x.GuideId.Equals(model.GuideId))
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return new();
        }

        public ExcursionGroupViewModel? GetElement(ExcursionGroupSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ExcursionGroupName) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new TravelAgencyDatabase();
            return context.ExcursionGroups
                .Include(x => x.User)
                .Include(x => x.Guide)
                .Include(x => x.Tours)
                .ThenInclude(x => x.Tour)
                .Include(x => x.Places)
                .ThenInclude(x => x.Place)
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.ExcursionGroupName) && x.ExcursionGroupName == model.ExcursionGroupName) ||
                                (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }

        public ExcursionGroupViewModel? Insert(ExcursionGroupBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var newExcursionGroup = ExcursionGroup.Create(context, model);
            if (newExcursionGroup == null)
            {
                return null;
            }
            context.ExcursionGroups.Add(newExcursionGroup);
            context.SaveChanges();
            return newExcursionGroup.GetViewModel;
        }

        public ExcursionGroupViewModel? Update(ExcursionGroupBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var excursionGroup = context.ExcursionGroups.FirstOrDefault(rec => rec.Id == model.Id);
                if (excursionGroup == null)
                {
                    return null;
                }
                excursionGroup.Update(model);
                context.SaveChanges();
                excursionGroup.UpdateTours(context, model);
                excursionGroup.UpdatePlaces(context, model);
                transaction.Commit();
                return excursionGroup.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public ExcursionGroupViewModel? Delete(ExcursionGroupBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var element = context.ExcursionGroups
                .Include(x => x.Tours)
                .Include(x => x.Places)
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.ExcursionGroups.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
