﻿using Microsoft.EntityFrameworkCore;
using TravelAgencyContracts.BindingModels;
using TravelAgencyContracts.SearchModels;
using TravelAgencyContracts.StoragesContracts;
using TravelAgencyContracts.ViewModels;
using TravelAgencyDatabaseImplement.Models;

namespace TravelAgencyDatabaseImplement.Implements
{
    public class ExcursionStorage : IExcursionStorage
    {
        public List<ExcursionViewModel> GetFullList()
        {
            using var context = new TravelAgencyDatabase();
            return context.Excursions
                    .Include(x => x.User)
                    .Include(x => x.Place)
                    .Include(x => x.Tours)
                    .ThenInclude(x => x.Tour)
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<ExcursionViewModel> GetFilteredList(ExcursionSearchModel model)
        {
            using var context = new TravelAgencyDatabase();
            if (!string.IsNullOrEmpty(model.ExcursionName))
            {
                return context.Excursions
                    .Include(x => x.User)
                    .Include(x => x.Place)
                    .Include(x => x.Tours)
                    .ThenInclude(x => x.Tour)
                    .Where(x => x.ExcursionName.Contains(model.ExcursionName))
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            if (model.UserId.HasValue)
            {
                return context.Excursions
                    .Include(x => x.User)
                    .Include(x => x.Place)
                    .Include(x => x.Tours)
                    .ThenInclude(x => x.Tour)
                    .Where(x => x.UserId.Equals(model.UserId))
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            if (model.PlaceId.HasValue)
            {
                return context.Excursions
                    .Include(x => x.User)
                    .Include(x => x.Place)
                    .Include(x => x.Tours)
                    .ThenInclude(x => x.Tour)
                    .Where(x => x.PlaceId.Equals(model.PlaceId))
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return new();
        }

        public ExcursionViewModel? GetElement(ExcursionSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ExcursionName) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new TravelAgencyDatabase();
            return context.Excursions
                .Include(x => x.User)
                .Include(x => x.Place)
                .Include(x => x.Tours)
                .ThenInclude(x => x.Tour)
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.ExcursionName) && x.ExcursionName == model.ExcursionName) ||
                                (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }

        public ExcursionViewModel? Insert(ExcursionBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var newExcursion = Excursion.Create(context, model);
            if (newExcursion == null)
            {
                return null;
            }
            context.Excursions.Add(newExcursion);
            context.SaveChanges();
            return newExcursion.GetViewModel;
        }

        public ExcursionViewModel? Update(ExcursionBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var excursion = context.Excursions.FirstOrDefault(rec => rec.Id == model.Id);
                if (excursion == null)
                {
                    return null;
                }
                excursion.Update(model);
                context.SaveChanges();
                excursion.UpdateTours(context, model);
                transaction.Commit();
                return excursion.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public ExcursionViewModel? Delete(ExcursionBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var element = context.Excursions
                .Include(x => x.Tours)
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Excursions.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
