﻿using TravelAgencyContracts.BindingModels;
using TravelAgencyContracts.SearchModels;
using TravelAgencyContracts.StoragesContracts;
using TravelAgencyContracts.ViewModels;
using TravelAgencyDatabaseImplement.Models;

namespace TravelAgencyDatabaseImplement.Implements
{
    public class GuideStorage : IGuideStorage
    {
        public List<GuideViewModel> GetFullList()
        {
            using var context = new TravelAgencyDatabase();
            return context.Guides
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<GuideViewModel> GetFilteredList(GuideSearchModel model)
        {
            if (string.IsNullOrEmpty(model.GuideFIO))
            {
                return new();
            }
            using var context = new TravelAgencyDatabase();
            return context.Guides
                    .Where(x => x.GuideFIO.Contains(model.GuideFIO))
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public GuideViewModel? GetElement(GuideSearchModel model)
        {
            using var context = new TravelAgencyDatabase();
            if (model.Id.HasValue)
            {
                return context.Guides
                    .FirstOrDefault(x => x.Id.Equals(model.Id))
                    ?.GetViewModel;
            }
            if (!string.IsNullOrEmpty(model.Email))
            {
                return context.Guides
                    .FirstOrDefault(x => x.Email.Equals(model.Email))
                    ?.GetViewModel;
            }
            if (!string.IsNullOrEmpty(model.PhoneNumber))
            {
                return context.Guides
                    .FirstOrDefault(x => x.PhoneNumber.Equals(model.PhoneNumber))
                    ?.GetViewModel;
            }
            return null;
        }

        public GuideViewModel? Insert(GuideBindingModel model)
        {
            var newGuide = Guide.Create(model);
            if (newGuide == null)
            {
                return null;
            }
            using var context = new TravelAgencyDatabase();
            context.Guides.Add(newGuide);
            context.SaveChanges();
            return newGuide.GetViewModel;
        }

        public GuideViewModel? Update(GuideBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var client = context.Guides.FirstOrDefault(x => x.Id == model.Id);
            if (client == null)
            {
                return null;
            }
            client.Update(model);
            context.SaveChanges();
            return client.GetViewModel;
        }

        public GuideViewModel? Delete(GuideBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var element = context.Guides.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                context.Guides.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
