﻿using Microsoft.EntityFrameworkCore;
using TravelAgencyContracts.BindingModels;
using TravelAgencyContracts.SearchModels;
using TravelAgencyContracts.StoragesContracts;
using TravelAgencyContracts.ViewModels;
using TravelAgencyDatabaseImplement.Models;

namespace TravelAgencyDatabaseImplement.Implements
{
    public class PlaceStorage : IPlaceStorage
    {
        public List<PlaceViewModel> GetFullList()
        {
            using var context = new TravelAgencyDatabase();
            return context.Places
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<PlaceViewModel> GetFilteredList(PlaceSearchModel model)
        {
            using var context = new TravelAgencyDatabase();
            if (!string.IsNullOrEmpty(model.PlaceName))
            {
                return context.Places
                    .Where(x => x.PlaceName.Contains(model.PlaceName))
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return new();
        }

        public PlaceViewModel? GetElement(PlaceSearchModel model)
        {
            using var context = new TravelAgencyDatabase();
            if (model.Id.HasValue)
            {
                return context.Places
                    .FirstOrDefault(x => x.Id.Equals(model.Id))
                    ?.GetViewModel;
            }
            if (!string.IsNullOrEmpty(model.PlaceName))
            {
                return context.Places
                    .FirstOrDefault(x => x.PlaceName.Equals(model.PlaceName))
                    ?.GetViewModel;
            }
            if (!string.IsNullOrEmpty(model.PlaceAddress))
            {
                return context.Places
                    .FirstOrDefault(x => x.PlaceAddress.Equals(model.PlaceAddress))
                    ?.GetViewModel;
            }
            return null;
        }

        public PlaceViewModel? Insert(PlaceBindingModel model)
        {
            var newPlace = Place.Create(model);
            if (newPlace == null)
            {
                return null;
            }
            using var context = new TravelAgencyDatabase();
            context.Places.Add(newPlace);
            context.SaveChanges();
            return context.Places
                      .FirstOrDefault(x => x.Id == newPlace.Id)
                      ?.GetViewModel;
        }

        public PlaceViewModel? Update(PlaceBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var place = context.Places.FirstOrDefault(x => x.Id == model.Id);
            if (place == null)
            {
                return null;
            }
            place.Update(model);
            context.SaveChanges();
            return context.Places
                         .FirstOrDefault(x => x.Id == model.Id)
                         ?.GetViewModel;
        }

        public PlaceViewModel? Delete(PlaceBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var element = context.Places.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                var deletedElement = context.Places
                                           .FirstOrDefault(x => x.Id == model.Id)
                                           ?.GetViewModel;
                context.Places.Remove(element);
                context.SaveChanges();
                return deletedElement;
            }
            return null;
        }
    }
}
