﻿using Microsoft.EntityFrameworkCore;
using TravelAgencyContracts.BindingModels;
using TravelAgencyContracts.SearchModels;
using TravelAgencyContracts.StoragesContracts;
using TravelAgencyContracts.ViewModels;
using TravelAgencyDatabaseImplement.Models;

namespace TravelAgencyDatabaseImplement.Implements
{
    public class TourStorage : ITourStorage
    {
        public List<TourViewModel> GetFullList()
        {
            using var context = new TravelAgencyDatabase();
            return context.Tours
                    .Include(x => x.User)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<TourViewModel> GetFilteredList(TourSearchModel model)
        {
            using var context = new TravelAgencyDatabase();
            var tours = context.Tours.Include(x => x.User)
                    .Select(x => x.GetViewModel)
                    .ToList();

            if (!string.IsNullOrEmpty(model.TourName))
            {
                tours = tours.Where(x => x.TourName.Contains(model.TourName)).ToList();
            }
            if (model.UserId.HasValue)
            {
                tours = tours.Where(x => x.UserId.Equals(model.UserId)).ToList();
            }

            if (model.DateFrom.HasValue && model.DateTo.HasValue)
            {
                tours = tours.Where(x => x.TourDate >= model.DateFrom && x.TourDate <= model.DateTo).ToList();
            }
            return tours ?? new();
        }

        public TourViewModel? GetElement(TourSearchModel model)
        {
            if (string.IsNullOrEmpty(model.TourName) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new TravelAgencyDatabase();
            return context.Tours.Include(x => x.User)
                    .FirstOrDefault(x => (!string.IsNullOrEmpty(model.TourName) && x.TourName == model.TourName) ||
                                        (model.Id.HasValue && x.Id == model.Id))
                    ?.GetViewModel;
        }

        public TourViewModel? Insert(TourBindingModel model)
        {
            var newTour = Tour.Create(model);
            if (newTour == null)
            {
                return null;
            }
            using var context = new TravelAgencyDatabase();
            context.Tours.Add(newTour);
            context.SaveChanges();
            return context.Tours
                      .Include(x => x.User)
                      .FirstOrDefault(x => x.Id == newTour.Id)
                      ?.GetViewModel;
        }

        public TourViewModel? Update(TourBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var tour = context.Tours.FirstOrDefault(x => x.Id == model.Id);
            if (tour == null)
            {
                return null;
            }
            tour.Update(model);
            context.SaveChanges();
            return context.Tours
                         .Include(x => x.User)
                         .FirstOrDefault(x => x.Id == model.Id)
                         ?.GetViewModel;
        }

        public TourViewModel? Delete(TourBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var element = context.Tours.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                var deletedElement = context.Tours
                                           .Include(x => x.User)
                                           .FirstOrDefault(x => x.Id == model.Id)
                                           ?.GetViewModel;
                context.Tours.Remove(element);
                context.SaveChanges();
                return deletedElement;
            }
            return null;
        }
    }
}
