﻿using Microsoft.EntityFrameworkCore;
using TravelAgencyContracts.BindingModels;
using TravelAgencyContracts.SearchModels;
using TravelAgencyContracts.StoragesContracts;
using TravelAgencyContracts.ViewModels;
using TravelAgencyDatabaseImplement.Models;

namespace TravelAgencyDatabaseImplement.Implements
{
    public class TripStorage : ITripStorage
    {
        public List<TripViewModel> GetFullList()
        {
            using var context = new TravelAgencyDatabase();
            return context.Trips
                    .Include(x => x.Guide)
                    .Include(x => x.Places)
                    .ThenInclude(x => x.Place)
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<TripViewModel> GetFilteredList(TripSearchModel model)
        {
            using var context = new TravelAgencyDatabase();
            if (!string.IsNullOrEmpty(model.TripName))
            {
                return context.Trips
                    .Include(x => x.Guide)
                    .Include(x => x.Places)
                    .ThenInclude(x => x.Place)
                    .Where(x => x.TripName.Contains(model.TripName))
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            if (model.GuideId.HasValue)
            {
                return context.Trips
                    .Include(x => x.Guide)
                    .Include(x => x.Places)
                    .ThenInclude(x => x.Place)
                    .Where(x => x.GuideId.Equals(model.GuideId))
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return new();
        }

        public TripViewModel? GetElement(TripSearchModel model)
        {
            if (string.IsNullOrEmpty(model.TripName) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new TravelAgencyDatabase();
            return context.Trips
                .Include(x => x.Guide)
                .Include(x => x.Places)
                .ThenInclude(x => x.Place)
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.TripName) && x.TripName == model.TripName) ||
                                (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }

        public TripViewModel? Insert(TripBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var newTrip = Trip.Create(context, model);
            if (newTrip == null)
            {
                return null;
            }
            context.Trips.Add(newTrip);
            context.SaveChanges();
            return newTrip.GetViewModel;
        }

        public TripViewModel? Update(TripBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var trip = context.Trips.FirstOrDefault(rec => rec.Id == model.Id);
                if (trip == null)
                {
                    return null;
                }
                trip.Update(model);
                context.SaveChanges();
                trip.UpdatePlaces(context, model);
                transaction.Commit();
                return trip.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public TripViewModel? Delete(TripBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var element = context.Trips
                .Include(x => x.Places)
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Trips.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
