﻿using TravelAgencyContracts.BindingModels;
using TravelAgencyContracts.SearchModels;
using TravelAgencyContracts.StoragesContracts;
using TravelAgencyContracts.ViewModels;
using TravelAgencyDatabaseImplement.Models;

namespace TravelAgencyDatabaseImplement.Implements
{
    public class UserStorage : IUserStorage
    {
        public List<UserViewModel> GetFullList()
        {
            using var context = new TravelAgencyDatabase();
            return context.Users
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<UserViewModel> GetFilteredList(UserSearchModel model)
        {
            if (string.IsNullOrEmpty(model.UserFIO))
            {
                return new();
            }
            using var context = new TravelAgencyDatabase();
            return context.Users
                    .Where(x => x.UserFIO.Contains(model.UserFIO))
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public UserViewModel? GetElement(UserSearchModel model)
        {
            using var context = new TravelAgencyDatabase();
            if (model.Id.HasValue)
            {
                return context.Users
                    .FirstOrDefault(x => x.Id.Equals(model.Id))
                    ?.GetViewModel;
            }
            if (!string.IsNullOrEmpty(model.Email) && !string.IsNullOrEmpty(model.Password))
            {
                return context.Users
                    .FirstOrDefault(x => x.Email.Equals(model.Email) && x.Password.Equals(model.Password))
                    ?.GetViewModel;
            }
            if (!string.IsNullOrEmpty(model.Email) && !string.IsNullOrEmpty(model.PhoneNumber))
            {
                return context.Users
                    .FirstOrDefault(x => x.Email.Equals(model.Email) || x.PhoneNumber.Equals(model.PhoneNumber))
                    ?.GetViewModel;
            }
            if (!string.IsNullOrEmpty(model.Email))
            {
                return context.Users
                    .FirstOrDefault(x => x.Email.Equals(model.Email))
                    ?.GetViewModel;
            }
            if (!string.IsNullOrEmpty(model.PhoneNumber))
            {
                return context.Users
                    .FirstOrDefault(x => x.PhoneNumber.Equals(model.PhoneNumber))
                    ?.GetViewModel;
            }
            return null;
        }

        public UserViewModel? Insert(UserBindingModel model)
        {
            var newUser = User.Create(model);
            if (newUser == null)
            {
                return null;
            }
            using var context = new TravelAgencyDatabase();
            context.Users.Add(newUser);
            context.SaveChanges();
            return newUser.GetViewModel;
        }

        public UserViewModel? Update(UserBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var user = context.Users.FirstOrDefault(x => x.Id == model.Id);
            if (user == null)
            {
                return null;
            }
            user.Update(model);
            context.SaveChanges();
            return user.GetViewModel;
        }

        public UserViewModel? Delete(UserBindingModel model)
        {
            using var context = new TravelAgencyDatabase();
            var element = context.Users.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                context.Users.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
